@@ -1,15 +1,14 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
 use crate::sys::component::Component;
+use crate::sys::cpu::*;
 use crate::sys::disk::*;
-use crate::sys::ffi;
 use crate::sys::network::Networks;
 use crate::sys::process::*;
-use crate::sys::processor::*;
-#[cfg(target_os = "macos")]
-use core_foundation_sys::base::{kCFAllocatorDefault, CFRelease};
 
-use crate::{LoadAvg, Pid, ProcessRefreshKind, ProcessorExt, RefreshKind, SystemExt, User};
+use crate::{
+    CpuExt, CpuRefreshKind, LoadAvg, Pid, ProcessRefreshKind, RefreshKind, SystemExt, User,
+};
 
 #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
 use crate::ProcessExt;
@@ -18,6 +17,8 @@ use std::cell::UnsafeCell;
 use std::collections::HashMap;
 use std::mem;
 use std::sync::Arc;
+use std::time::Duration;
+#[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
 use std::time::SystemTime;
 
 #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
@@ -28,6 +29,9 @@ use libc::{
     vm_statistics64, _SC_PAGESIZE,
 };
 
+#[cfg(not(any(target_os = "ios", feature = "apple-sandbox")))]
+use super::inner::component::Components;
+
 #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
 declare_signals! {
     c_int,
@@ -80,40 +84,19 @@ pub struct System {
     mem_available: u64,
     swap_total: u64,
     swap_free: u64,
-    global_processor: Processor,
-    processors: Vec<Processor>,
+    global_cpu: Cpu,
+    cpus: Vec<Cpu>,
     page_size_kb: u64,
-    components: Vec<Component>,
-    // Used to get CPU information, not supported on iOS, or inside the default macOS sandbox.
-    #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
-    connection: Option<ffi::io_connect_t>,
+    #[cfg(not(any(target_os = "ios", feature = "apple-sandbox")))]
+    components: Components,
     disks: Vec<Disk>,
     networks: Networks,
     port: mach_port_t,
     users: Vec<User>,
     boot_time: u64,
-    // Used to get disk information, to be more specific, it's needed by the
-    // DADiskCreateFromVolumePath function. Not supported on iOS.
-    #[cfg(target_os = "macos")]
-    session: ffi::SessionWrap,
     #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
     clock_info: Option<crate::sys::macos::system::SystemTimeInfo>,
-}
-
-impl Drop for System {
-    fn drop(&mut self) {
-        unsafe {
-            #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
-            if let Some(conn) = self.connection {
-                ffi::IOServiceClose(conn);
-            }
-
-            #[cfg(target_os = "macos")]
-            if !self.session.0.is_null() {
-                CFRelease(self.session.0 as _);
-            }
-        }
-    }
+    got_cpu_frequency: bool,
 }
 
 pub(crate) struct Wrap<'a>(pub UnsafeCell<&'a mut HashMap<Pid, Process>>);
@@ -121,16 +104,6 @@ pub(crate) struct Wrap<'a>(pub UnsafeCell<&'a mut HashMap<Pid, Process>>);
 unsafe impl<'a> Send for Wrap<'a> {}
 unsafe impl<'a> Sync for Wrap<'a> {}
 
-#[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
-impl System {
-    fn clear_procs(&mut self) {
-        use crate::sys::macos::process;
-
-        self.process_list
-            .retain(|_, proc_| process::has_been_updated(proc_));
-    }
-}
-
 fn boot_time() -> u64 {
     let mut boot_time = timeval {
         tv_sec: 0,
@@ -156,6 +129,7 @@ fn boot_time() -> u64 {
     }
 }
 
+#[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
 fn get_now() -> u64 {
     SystemTime::now()
         .duration_since(SystemTime::UNIX_EPOCH)
@@ -166,6 +140,7 @@ fn get_now() -> u64 {
 impl SystemExt for System {
     const IS_SUPPORTED: bool = true;
     const SUPPORTED_SIGNALS: &'static [Signal] = supported_signals();
+    const MINIMUM_CPU_UPDATE_INTERVAL: Duration = Duration::from_millis(200);
 
     fn new_with_specifics(refreshes: RefreshKind) -> System {
         unsafe {
@@ -178,27 +153,25 @@ impl SystemExt for System {
                 mem_available: 0,
                 swap_total: 0,
                 swap_free: 0,
-                global_processor: Processor::new(
+                global_cpu: Cpu::new(
                     "0".to_owned(),
-                    Arc::new(ProcessorData::new(std::ptr::null_mut(), 0)),
+                    Arc::new(CpuData::new(std::ptr::null_mut(), 0)),
                     0,
                     String::new(),
                     String::new(),
                 ),
-                processors: Vec::new(),
-                page_size_kb: sysconf(_SC_PAGESIZE) as u64 / 1_000,
-                components: Vec::with_capacity(2),
-                #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
-                connection: get_io_service_connection(),
+                cpus: Vec::new(),
+                page_size_kb: sysconf(_SC_PAGESIZE) as _,
+                #[cfg(not(any(target_os = "ios", feature = "apple-sandbox")))]
+                components: Components::new(),
                 disks: Vec::with_capacity(1),
                 networks: Networks::new(),
                 port,
                 users: Vec::new(),
                 boot_time: boot_time(),
-                #[cfg(target_os = "macos")]
-                session: ffi::SessionWrap(::std::ptr::null_mut()),
                 #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
                 clock_info: crate::sys::macos::system::SystemTimeInfo::new(port),
+                got_cpu_frequency: false,
             };
             s.refresh_specifics(refreshes);
             s
@@ -219,8 +192,8 @@ impl SystemExt for System {
                 &mut xs as *mut _ as *mut c_void,
                 &mut mib,
             ) {
-                self.swap_total = xs.xsu_total / 1_000;
-                self.swap_free = xs.xsu_avail / 1_000;
+                self.swap_total = xs.xsu_total;
+                self.swap_free = xs.xsu_avail;
             }
             // get ram info
             if self.mem_total < 1 {
@@ -231,7 +204,6 @@ impl SystemExt for System {
                     &mut self.mem_total as *mut u64 as *mut c_void,
                     &mut mib,
                 );
-                self.mem_total /= 1_000;
             }
             let mut count: u32 = libc::HOST_VM_INFO64_COUNT as _;
             let mut stat = mem::zeroed::<vm_statistics64>();
@@ -250,14 +222,12 @@ impl SystemExt for System {
                 //  * used to hold data that was read speculatively from disk but
                 //  * haven't actually been used by anyone so far.
                 //  */
-                self.mem_available = self.mem_total
-                    - (u64::from(stat.active_count)
-                        + u64::from(stat.inactive_count)
-                        + u64::from(stat.wire_count)
-                        + u64::from(stat.speculative_count)
-                        - u64::from(stat.purgeable_count))
-                        * self.page_size_kb;
-                self.mem_free = u64::from(stat.free_count) * self.page_size_kb;
+                self.mem_available = u64::from(stat.free_count)
+                    .saturating_add(u64::from(stat.inactive_count))
+                    .saturating_mul(self.page_size_kb);
+                self.mem_free = u64::from(stat.free_count)
+                    .saturating_sub(u64::from(stat.speculative_count))
+                    .saturating_mul(self.page_size_kb);
             }
         }
     }
@@ -265,45 +235,38 @@ impl SystemExt for System {
     #[cfg(any(target_os = "ios", feature = "apple-sandbox"))]
     fn refresh_components_list(&mut self) {}
 
-    #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
+    #[cfg(not(any(target_os = "ios", feature = "apple-sandbox")))]
     fn refresh_components_list(&mut self) {
-        if let Some(con) = self.connection {
-            self.components.clear();
-            // getting CPU critical temperature
-            let critical_temp = crate::apple::component::get_temperature(
-                con,
-                &['T' as i8, 'C' as i8, '0' as i8, 'D' as i8, 0],
-            );
-
-            for (id, v) in crate::apple::component::COMPONENTS_TEMPERATURE_IDS.iter() {
-                if let Some(c) = Component::new((*id).to_owned(), None, critical_temp, v, con) {
-                    self.components.push(c);
-                }
-            }
-        }
+        self.components.refresh();
     }
 
-    fn refresh_cpu(&mut self) {
-        let processors = &mut self.processors;
-        if processors.is_empty() {
-            init_processors(self.port, processors, &mut self.global_processor);
-        } else {
-            update_processor_usage(
-                self.port,
-                &mut self.global_processor,
-                |proc_data, cpu_info| {
-                    let mut percentage = 0f32;
-                    let mut offset = 0;
-                    for proc_ in processors.iter_mut() {
-                        let cpu_usage = compute_processor_usage(proc_, cpu_info, offset);
-                        proc_.update(cpu_usage, Arc::clone(&proc_data));
-                        percentage += proc_.cpu_usage();
+    fn refresh_cpu_specifics(&mut self, refresh_kind: CpuRefreshKind) {
+        let cpus = &mut self.cpus;
+        if cpus.is_empty() {
+            init_cpus(self.port, cpus, &mut self.global_cpu, refresh_kind);
+            self.got_cpu_frequency = refresh_kind.frequency();
+            return;
+        }
+        if refresh_kind.frequency() && !self.got_cpu_frequency {
+            let frequency = get_cpu_frequency();
+            for proc_ in cpus.iter_mut() {
+                proc_.set_frequency(frequency);
+            }
+            self.got_cpu_frequency = true;
+        }
+        if refresh_kind.cpu_usage() {
+            update_cpu_usage(self.port, &mut self.global_cpu, |proc_data, cpu_info| {
+                let mut percentage = 0f32;
+                let mut offset = 0;
+                for proc_ in cpus.iter_mut() {
+                    let cpu_usage = compute_usage_of_cpu(proc_, cpu_info, offset);
+                    proc_.update(cpu_usage, Arc::clone(&proc_data));
+                    percentage += proc_.cpu_usage();
 
-                        offset += libc::CPU_STATE_MAX as isize;
-                    }
-                    (percentage, processors.len())
-                },
-            );
+                    offset += libc::CPU_STATE_MAX as isize;
+                }
+                (percentage, cpus.len())
+            });
         }
     }
 
@@ -340,6 +303,7 @@ impl SystemExt for System {
                             time_interval,
                             now,
                             refresh_kind,
+                            false,
                         ) {
                             Ok(x) => x,
                             _ => None,
@@ -350,7 +314,8 @@ impl SystemExt for System {
             entries.into_iter().for_each(|entry| {
                 self.process_list.insert(entry.pid(), entry);
             });
-            self.clear_procs();
+            self.process_list
+                .retain(|_, proc_| std::mem::replace(&mut proc_.updated, false));
         }
     }
 
@@ -361,10 +326,14 @@ impl SystemExt for System {
 
     #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
     fn refresh_process_specifics(&mut self, pid: Pid, refresh_kind: ProcessRefreshKind) -> bool {
-        let now = get_now();
+        let mut time_interval = None;
         let arg_max = get_arg_max();
-        let port = self.port;
-        let time_interval = self.clock_info.as_mut().map(|c| c.get_time_interval(port));
+        let now = get_now();
+
+        if refresh_kind.cpu() {
+            let port = self.port;
+            time_interval = self.clock_info.as_mut().map(|c| c.get_time_interval(port));
+        }
         match {
             let wrap = Wrap(UnsafeCell::new(&mut self.process_list));
             update_process(
@@ -374,6 +343,7 @@ impl SystemExt for System {
                 time_interval,
                 now,
                 refresh_kind,
+                true,
             )
         } {
             Ok(Some(p)) => {
@@ -385,17 +355,8 @@ impl SystemExt for System {
         }
     }
 
-    #[cfg(target_os = "ios")]
-    fn refresh_disks_list(&mut self) {}
-
-    #[cfg(target_os = "macos")]
     fn refresh_disks_list(&mut self) {
-        unsafe {
-            if self.session.0.is_null() {
-                self.session.0 = ffi::DASessionCreate(kCFAllocatorDefault as _);
-            }
-            self.disks = get_disks(self.session.0);
-        }
+        self.disks = unsafe { get_disks() };
     }
 
     fn refresh_users_list(&mut self) {
@@ -414,12 +375,12 @@ impl SystemExt for System {
         self.process_list.get(&pid)
     }
 
-    fn global_processor_info(&self) -> &Processor {
-        &self.global_processor
+    fn global_cpu_info(&self) -> &Cpu {
+        &self.global_cpu
     }
 
-    fn processors(&self) -> &[Processor] {
-        &self.processors
+    fn cpus(&self) -> &[Cpu] {
+        &self.cpus
     }
 
     fn physical_core_count(&self) -> Option<usize> {
@@ -475,12 +436,24 @@ impl SystemExt for System {
         self.swap_total - self.swap_free
     }
 
+    #[cfg(not(any(target_os = "ios", feature = "apple-sandbox")))]
     fn components(&self) -> &[Component] {
-        &self.components
+        &self.components.inner
     }
 
+    #[cfg(any(target_os = "ios", feature = "apple-sandbox"))]
+    fn components(&self) -> &[Component] {
+        &[]
+    }
+
+    #[cfg(not(any(target_os = "ios", feature = "apple-sandbox")))]
     fn components_mut(&mut self) -> &mut [Component] {
-        &mut self.components
+        &mut self.components.inner
+    }
+
+    #[cfg(any(target_os = "ios", feature = "apple-sandbox"))]
+    fn components_mut(&mut self) -> &mut [Component] {
+        &mut []
     }
 
     fn disks(&self) -> &[Disk] {
@@ -491,6 +464,13 @@ impl SystemExt for System {
         &mut self.disks
     }
 
+    fn sort_disks_by<F>(&mut self, compare: F)
+    where
+        F: FnMut(&Disk, &Disk) -> std::cmp::Ordering,
+    {
+        self.disks.sort_unstable_by(compare);
+    }
+
     fn uptime(&self) -> u64 {
         unsafe {
             let csec = libc::time(::std::ptr::null_mut());
@@ -582,7 +562,7 @@ impl SystemExt for System {
                 && size > 0
             {
                 // now create a buffer with the size and get the real value
-                let mut buf = vec![0_u8; size as usize];
+                let mut buf = vec![0_u8; size as _];
 
                 if get_sys_value_by_name(
                     b"kern.osproductversion\0",
@@ -605,6 +585,10 @@ impl SystemExt for System {
             }
         }
     }
+
+    fn distribution_id(&self) -> String {
+        std::env::consts::OS.to_owned()
+    }
 }
 
 impl Default for System {
@@ -613,43 +597,6 @@ impl Default for System {
     }
 }
 
-// code from https://github.com/Chris911/iStats
-// Not supported on iOS, or in the default macOS
-#[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
-fn get_io_service_connection() -> Option<ffi::io_connect_t> {
-    let mut master_port: mach_port_t = 0;
-    let mut iterator: ffi::io_iterator_t = 0;
-
-    unsafe {
-        ffi::IOMasterPort(libc::MACH_PORT_NULL, &mut master_port);
-
-        let matching_dictionary = ffi::IOServiceMatching(b"AppleSMC\0".as_ptr() as *const i8);
-        let result =
-            ffi::IOServiceGetMatchingServices(master_port, matching_dictionary, &mut iterator);
-        if result != ffi::KIO_RETURN_SUCCESS {
-            sysinfo_debug!("Error: IOServiceGetMatchingServices() = {}", result);
-            return None;
-        }
-
-        let device = ffi::IOIteratorNext(iterator);
-        ffi::IOObjectRelease(iterator);
-        if device == 0 {
-            sysinfo_debug!("Error: no SMC found");
-            return None;
-        }
-
-        let mut conn = 0;
-        let result = ffi::IOServiceOpen(device, libc::mach_task_self(), 0, &mut conn);
-        ffi::IOObjectRelease(device);
-        if result != ffi::KIO_RETURN_SUCCESS {
-            sysinfo_debug!("Error: IOServiceOpen() = {}", result);
-            return None;
-        }
-
-        Some(conn)
-    }
-}
-
 #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
 fn get_arg_max() -> usize {
     let mut mib = [libc::CTL_KERN, libc::KERN_ARGMAX];
@@ -721,7 +668,7 @@ fn get_system_info(value: c_int, default: Option<&str>) -> Option<String> {
             default.map(|s| s.to_owned())
         } else {
             // set the buffer to the correct size
-            let mut buf = vec![0_u8; size as usize];
+            let mut buf = vec![0_u8; size as _];
 
             if sysctl(
                 mib.as_mut_ptr(),
