@@ -2,7 +2,7 @@
 
 use crate::{NetworkData, Networks, NetworksExt, UserExt};
 
-use std::convert::From;
+use std::convert::{From, TryFrom};
 use std::fmt;
 use std::str::FromStr;
 
@@ -14,7 +14,7 @@ use std::str::FromStr;
 /// let p = Pid::from_u32(0);
 /// let value: u32 = p.as_u32();
 /// ```
-pub trait PidExt<T>: Copy + From<T> + FromStr + fmt::Display {
+pub trait PidExt: Copy + From<usize> + FromStr + fmt::Display {
     /// Allows to convert [`Pid`][crate::Pid] into [`u32`].
     ///
     /// ```
@@ -41,19 +41,19 @@ macro_rules! pid_decl {
         #[repr(transparent)]
         pub struct Pid(pub(crate) $typ);
 
-        impl From<$typ> for Pid {
-            fn from(v: $typ) -> Self {
-                Self(v)
+        impl From<usize> for Pid {
+            fn from(v: usize) -> Self {
+                Self(v as _)
             }
         }
-        impl From<Pid> for $typ {
+        impl From<Pid> for usize {
             fn from(v: Pid) -> Self {
-                v.0
+                v.0 as _
             }
         }
-        impl PidExt<$typ> for Pid {
+        impl PidExt for Pid {
             fn as_u32(self) -> u32 {
-                self.0 as u32
+                self.0 as _
             }
             fn from_u32(v: u32) -> Self {
                 Self(v as _)
@@ -93,9 +93,12 @@ cfg_if::cfg_if! {
 }
 
 macro_rules! impl_get_set {
-    ($ty_name:ident, $name:ident, $with:ident, $without:ident) => {
-        #[doc = concat!("Returns the value of the \"", stringify!($name), "\" refresh kind.
-
+    ($ty_name:ident, $name:ident, $with:ident, $without:ident $(, $extra_doc:literal)? $(,)?) => {
+        #[doc = concat!("Returns the value of the \"", stringify!($name), "\" refresh kind.")]
+        $(#[doc = concat!("
+", $extra_doc, "
+")])?
+        #[doc = concat!("
 ```
 use sysinfo::", stringify!($ty_name), ";
 
@@ -146,10 +149,68 @@ assert_eq!(r.", stringify!($name), "(), false);
             self
         }
     };
+
+    ($ty_name:ident, $name:ident, $with:ident, $without:ident, $typ:ty $(,)?) => {
+        #[doc = concat!("Returns the value of the \"", stringify!($name), "\" refresh kind.
+
+```
+use sysinfo::{", stringify!($ty_name), ", ", stringify!($typ), "};
+
+let r = ", stringify!($ty_name), "::new();
+assert_eq!(r.", stringify!($name), "().is_some(), false);
+
+let r = r.with_", stringify!($name), "(", stringify!($typ), "::everything());
+assert_eq!(r.", stringify!($name), "().is_some(), true);
+
+let r = r.without_", stringify!($name), "();
+assert_eq!(r.", stringify!($name), "().is_some(), false);
+```")]
+        pub fn $name(&self) -> Option<$typ> {
+            self.$name
+        }
+
+        #[doc = concat!("Sets the value of the \"", stringify!($name), "\" refresh kind to `true`.
+
+```
+use sysinfo::{", stringify!($ty_name), ", ", stringify!($typ), "};
+
+let r = ", stringify!($ty_name), "::new();
+assert_eq!(r.", stringify!($name), "().is_some(), false);
+
+let r = r.with_", stringify!($name), "(", stringify!($typ), "::everything());
+assert_eq!(r.", stringify!($name), "().is_some(), true);
+```")]
+        #[must_use]
+        pub fn $with(mut self, kind: $typ) -> Self {
+            self.$name = Some(kind);
+            self
+        }
+
+        #[doc = concat!("Sets the value of the \"", stringify!($name), "\" refresh kind to `false`.
+
+```
+use sysinfo::", stringify!($ty_name), ";
+
+let r = ", stringify!($ty_name), "::everything();
+assert_eq!(r.", stringify!($name), "().is_some(), true);
+
+let r = r.without_", stringify!($name), "();
+assert_eq!(r.", stringify!($name), "().is_some(), false);
+```")]
+        #[must_use]
+        pub fn $without(mut self) -> Self {
+            self.$name = None;
+            self
+        }
+    };
 }
 
 /// Used to determine what you want to refresh specifically on the [`Process`] type.
 ///
+/// ⚠️ Just like all other refresh types, ruling out a refresh doesn't assure you that
+/// the information won't be retrieved if the information is accessible without needing
+/// extra computation.
+///
 /// ```
 /// use sysinfo::{ProcessExt, ProcessRefreshKind, System, SystemExt};
 ///
@@ -169,6 +230,7 @@ assert_eq!(r.", stringify!($name), "(), false);
 pub struct ProcessRefreshKind {
     cpu: bool,
     disk_usage: bool,
+    user: bool,
 }
 
 impl ProcessRefreshKind {
@@ -200,6 +262,7 @@ impl ProcessRefreshKind {
         Self {
             cpu: true,
             disk_usage: true,
+            user: true,
         }
     }
 
@@ -210,10 +273,82 @@ impl ProcessRefreshKind {
         with_disk_usage,
         without_disk_usage
     );
+    impl_get_set!(
+        ProcessRefreshKind,
+        user,
+        with_user,
+        without_user,
+        r#"This refresh is about `user_id` and `group_id`. Please note that it has an effect mostly
+on Windows as other platforms get this information alongside the Process information directly."#,
+    );
+}
+
+/// Used to determine what you want to refresh specifically on the [`Cpu`] type.
+///
+/// ⚠️ Just like all other refresh types, ruling out a refresh doesn't assure you that
+/// the information won't be retrieved if the information is accessible without needing
+/// extra computation.
+///
+/// ```
+/// use sysinfo::{CpuExt, CpuRefreshKind, System, SystemExt};
+///
+/// let mut system = System::new();
+///
+/// // We don't want to update all the CPU information.
+/// system.refresh_cpu_specifics(CpuRefreshKind::everything().without_frequency());
+///
+/// for cpu in system.cpus() {
+///     assert_eq!(cpu.frequency(), 0);
+/// }
+/// ```
+///
+/// [`Cpu`]: crate::Cpu
+#[derive(Clone, Copy, Debug, Default, PartialEq, Eq)]
+pub struct CpuRefreshKind {
+    cpu_usage: bool,
+    frequency: bool,
+}
+
+impl CpuRefreshKind {
+    /// Creates a new `CpuRefreshKind` with every refresh set to `false`.
+    ///
+    /// ```
+    /// use sysinfo::CpuRefreshKind;
+    ///
+    /// let r = CpuRefreshKind::new();
+    ///
+    /// assert_eq!(r.frequency(), false);
+    /// ```
+    pub fn new() -> Self {
+        Self::default()
+    }
+
+    /// Creates a new `CpuRefreshKind` with every refresh set to `true`.
+    ///
+    /// ```
+    /// use sysinfo::CpuRefreshKind;
+    ///
+    /// let r = CpuRefreshKind::everything();
+    ///
+    /// assert_eq!(r.frequency(), true);
+    /// ```
+    pub fn everything() -> Self {
+        Self {
+            cpu_usage: true,
+            frequency: true,
+        }
+    }
+
+    impl_get_set!(CpuRefreshKind, cpu_usage, with_cpu_usage, without_cpu_usage);
+    impl_get_set!(CpuRefreshKind, frequency, with_frequency, without_frequency);
 }
 
 /// Used to determine what you want to refresh specifically on the [`System`] type.
 ///
+/// ⚠️ Just like all other refresh types, ruling out a refresh doesn't assure you that
+/// the information won't be retrieved if the information is accessible without needing
+/// extra computation.
+///
 /// ```
 /// use sysinfo::{RefreshKind, System, SystemExt};
 ///
@@ -235,14 +370,14 @@ pub struct RefreshKind {
     disks_list: bool,
     disks: bool,
     memory: bool,
-    cpu: bool,
+    cpu: Option<CpuRefreshKind>,
     components: bool,
     components_list: bool,
     users_list: bool,
 }
 
 impl RefreshKind {
-    /// Creates a new `RefreshKind` with every refresh set to `false`.
+    /// Creates a new `RefreshKind` with every refresh set to `false`/`None`.
     ///
     /// ```
     /// use sysinfo::RefreshKind;
@@ -255,7 +390,7 @@ impl RefreshKind {
     /// assert_eq!(r.disks_list(), false);
     /// assert_eq!(r.disks(), false);
     /// assert_eq!(r.memory(), false);
-    /// assert_eq!(r.cpu(), false);
+    /// assert_eq!(r.cpu().is_some(), false);
     /// assert_eq!(r.components(), false);
     /// assert_eq!(r.components_list(), false);
     /// assert_eq!(r.users_list(), false);
@@ -264,7 +399,7 @@ impl RefreshKind {
         Self::default()
     }
 
-    /// Creates a new `RefreshKind` with every refresh set to `true`.
+    /// Creates a new `RefreshKind` with every refresh set to `true`/`Some(...)`.
     ///
     /// ```
     /// use sysinfo::RefreshKind;
@@ -277,7 +412,7 @@ impl RefreshKind {
     /// assert_eq!(r.disks_list(), true);
     /// assert_eq!(r.disks(), true);
     /// assert_eq!(r.memory(), true);
-    /// assert_eq!(r.cpu(), true);
+    /// assert_eq!(r.cpu().is_some(), true);
     /// assert_eq!(r.components(), true);
     /// assert_eq!(r.components_list(), true);
     /// assert_eq!(r.users_list(), true);
@@ -290,65 +425,20 @@ impl RefreshKind {
             disks: true,
             disks_list: true,
             memory: true,
-            cpu: true,
+            cpu: Some(CpuRefreshKind::everything()),
             components: true,
             components_list: true,
             users_list: true,
         }
     }
 
-    /// Returns the value of the "processes" refresh kind.
-    ///
-    /// ```
-    /// use sysinfo::{ProcessRefreshKind, RefreshKind};
-    ///
-    /// let r = RefreshKind::new();
-    /// assert_eq!(r.processes(), None);
-    ///
-    /// let r = r.with_processes(ProcessRefreshKind::everything());
-    /// assert_eq!(r.processes().is_some(), true);
-    ///
-    /// let r = r.without_processes();
-    /// assert_eq!(r.processes().is_some(), false);
-    /// ```
-    pub fn processes(&self) -> Option<ProcessRefreshKind> {
-        self.processes
-    }
-
-    /// Sets the value of the "processes" refresh kind.
-    ///
-    /// ```
-    /// use sysinfo::{ProcessRefreshKind, RefreshKind};
-    ///
-    /// let r = RefreshKind::new();
-    /// assert_eq!(r.processes().is_some(), false);
-    ///
-    /// let r = r.with_processes(ProcessRefreshKind::everything());
-    /// assert_eq!(r.processes().is_some(), true);
-    /// ```
-    #[must_use]
-    pub fn with_processes(mut self, refresh_kind: ProcessRefreshKind) -> Self {
-        self.processes = Some(refresh_kind);
-        self
-    }
-
-    /// Sets the value of the "processes" refresh kind to `None`.
-    ///
-    /// ```
-    /// use sysinfo::{ProcessRefreshKind, RefreshKind};
-    ///
-    /// let r = RefreshKind::everything();
-    /// assert_eq!(r.processes().is_some(), true);
-    ///
-    /// let r = r.without_processes();
-    /// assert_eq!(r.processes().is_some(), false);
-    /// ```
-    #[must_use]
-    pub fn without_processes(mut self) -> Self {
-        self.processes = None;
-        self
-    }
-
+    impl_get_set!(
+        RefreshKind,
+        processes,
+        with_processes,
+        without_processes,
+        ProcessRefreshKind
+    );
     impl_get_set!(RefreshKind, networks, with_networks, without_networks);
     impl_get_set!(
         RefreshKind,
@@ -359,7 +449,7 @@ impl RefreshKind {
     impl_get_set!(RefreshKind, disks, with_disks, without_disks);
     impl_get_set!(RefreshKind, disks_list, with_disks_list, without_disks_list);
     impl_get_set!(RefreshKind, memory, with_memory, without_memory);
-    impl_get_set!(RefreshKind, cpu, with_cpu, without_cpu);
+    impl_get_set!(RefreshKind, cpu, with_cpu, without_cpu, CpuRefreshKind);
     impl_get_set!(RefreshKind, components, with_components, without_components);
     impl_get_set!(
         RefreshKind,
@@ -419,7 +509,7 @@ impl<'a> IntoIterator for &'a Networks {
 ///     println!("{:?}: {:?}", disk.name(), disk.type_());
 /// }
 /// ```
-#[derive(Debug, PartialEq, Clone, Copy)]
+#[derive(Debug, PartialEq, Eq, Clone, Copy)]
 pub enum DiskType {
     /// HDD type.
     HDD,
@@ -436,7 +526,7 @@ pub enum DiskType {
 ///
 /// If you want the list of the supported signals on the current system, use
 /// [`SystemExt::SUPPORTED_SIGNALS`][crate::SystemExt::SUPPORTED_SIGNALS].
-#[derive(Clone, Copy, PartialEq, PartialOrd, Debug)]
+#[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Debug)]
 pub enum Signal {
     /// Hangup detected on controlling terminal or death of controlling process.
     Hangup,
@@ -574,9 +664,10 @@ pub struct LoadAvg {
 }
 
 macro_rules! xid {
-    ($(#[$outer:meta])+ $name:ident, $type:ty) => {
+    ($(#[$outer:meta])+ $name:ident, $type:ty $(, $trait:ty)?) => {
         $(#[$outer])+
-        #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Debug)]
+        #[repr(transparent)]
+        #[derive(Clone, PartialEq, Eq, Hash, PartialOrd, Ord, Debug)]
         pub struct $name(pub(crate) $type);
 
         impl std::ops::Deref for $name {
@@ -586,6 +677,51 @@ macro_rules! xid {
                 &self.0
             }
         }
+
+        $(
+        impl TryFrom<usize> for $name {
+            type Error = <$type as TryFrom<usize>>::Error;
+
+            fn try_from(t: usize) -> Result<Self, <$type as TryFrom<usize>>::Error> {
+                Ok(Self(<$type>::try_from(t)?))
+            }
+        }
+
+        impl $trait for $name {
+            type Err = <$type as FromStr>::Err;
+
+            fn from_str(t: &str) -> Result<Self, <$type as FromStr>::Err> {
+                Ok(Self(<$type>::from_str(t)?))
+            }
+        }
+        )?
+    };
+}
+
+macro_rules! uid {
+    ($type:ty$(, $trait:ty)?) => {
+        xid!(
+            /// A user id wrapping a platform specific type.
+            ///
+            /// ⚠️ On windows, `Uid` is actually wrapping a `Box<str>` due to some
+            /// Windows limitations around their users ID. For the time being, `Uid`
+            /// is the user name on this platform.
+            Uid,
+            $type
+            $(, $trait)?
+        );
+    };
+}
+
+macro_rules! gid {
+    ($type:ty) => {
+        xid!(
+            /// A group id wrapping a platform specific type.
+            #[derive(Copy)]
+            Gid,
+            $type,
+            FromStr
+        );
     };
 }
 
@@ -600,27 +736,29 @@ cfg_if::cfg_if! {
             target_os = "ios",
         )
     ))] {
-        xid!(
-            /// A user id wrapping a platform specific type.
-            Uid,
-            libc::uid_t
-        );
-        xid!(
-            /// A group id wrapping a platform specific type.
-            Gid,
-            libc::gid_t
-        );
+        uid!(libc::uid_t, FromStr);
+        gid!(libc::gid_t);
+    } else if #[cfg(windows)] {
+        uid!(Box<str>);
+        gid!(u32);
+        // Manual implementation outside of the macro...
+        impl FromStr for Uid {
+            type Err = <String as FromStr>::Err;
+
+            fn from_str(t: &str) -> Result<Self, <String as FromStr>::Err> {
+                Ok(Self(t.into()))
+            }
+        }
+        impl TryFrom<usize> for Uid {
+            type Error = <u32 as TryFrom<usize>>::Error;
+
+            fn try_from(t: usize) -> Result<Self, <u32 as TryFrom<usize>>::Error> {
+                Ok(Self(t.to_string().into_boxed_str()))
+            }
+        }
     } else {
-        xid!(
-            /// A user id wrapping a platform specific type.
-            Uid,
-            u32
-        );
-        xid!(
-            /// A group id wrapping a platform specific type.
-            Gid,
-            u32
-        );
+        uid!(u32, FromStr);
+        gid!(u32);
     }
 }
 
@@ -643,11 +781,11 @@ pub struct User {
 }
 
 impl UserExt for User {
-    fn uid(&self) -> Uid {
-        self.uid
+    fn id(&self) -> &Uid {
+        &self.uid
     }
 
-    fn gid(&self) -> Gid {
+    fn group_id(&self) -> Gid {
         self.gid
     }
 
@@ -682,7 +820,7 @@ impl UserExt for User {
 ///     );
 /// }
 /// ```
-#[derive(Debug, Default, Clone, Copy, PartialEq, PartialOrd)]
+#[derive(Debug, Default, Clone, Copy, PartialEq, Eq, PartialOrd)]
 pub struct DiskUsage {
     /// Total number of written bytes.
     pub total_written_bytes: u64,
@@ -695,7 +833,7 @@ pub struct DiskUsage {
 }
 
 /// Enum describing the different status of a process.
-#[derive(Clone, Copy, Debug, PartialEq)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq)]
 pub enum ProcessStatus {
     /// ## Linux/FreeBSD
     ///
@@ -843,14 +981,81 @@ pub fn get_current_pid() -> Result<Pid, &'static str> {
     inner()
 }
 
+/// MAC address for network interface.
+///
+/// It is returned by [`NetworkExt::mac_address`][crate::NetworkExt::mac_address].
+#[derive(PartialEq, Eq, Copy, Clone, Debug)]
+pub struct MacAddr(pub [u8; 6]);
+
+impl MacAddr {
+    /// A `MacAddr` with all bytes set to `0`.
+    pub const UNSPECIFIED: Self = MacAddr([0; 6]);
+
+    /// Checks if this `MacAddr` has all bytes equal to `0`.
+    pub fn is_unspecified(&self) -> bool {
+        self == &MacAddr::UNSPECIFIED
+    }
+}
+
+impl fmt::Display for MacAddr {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        let data = &self.0;
+        write!(
+            f,
+            "{:02x}:{:02x}:{:02x}:{:02x}:{:02x}:{:02x}",
+            data[0], data[1], data[2], data[3], data[4], data[5],
+        )
+    }
+}
+
 #[cfg(test)]
 mod tests {
-    use super::ProcessStatus;
+    use super::{MacAddr, ProcessStatus};
 
-    // This test only exists to ensure that the `Display` trait is implemented on the
+    // This test only exists to ensure that the `Display` and `Debug` traits are implemented on the
     // `ProcessStatus` enum on all targets.
     #[test]
     fn check_display_impl_process_status() {
         println!("{} {:?}", ProcessStatus::Parked, ProcessStatus::Idle);
     }
+
+    // Ensure that the `Display` and `Debug` traits are implemented on the `MacAddr` struct
+    #[test]
+    fn check_display_impl_mac_address() {
+        println!(
+            "{} {:?}",
+            MacAddr([0x1, 0x2, 0x3, 0x4, 0x5, 0x6]),
+            MacAddr([0xa, 0xb, 0xc, 0xd, 0xe, 0xf])
+        );
+    }
+
+    #[test]
+    fn check_mac_address_is_unspecified_true() {
+        assert!(MacAddr::UNSPECIFIED.is_unspecified());
+        assert!(MacAddr([0; 6]).is_unspecified());
+    }
+
+    #[test]
+    fn check_mac_address_is_unspecified_false() {
+        assert!(!MacAddr([1, 2, 3, 4, 5, 6]).is_unspecified());
+    }
+
+    // This test exists to ensure that the `TryFrom<usize>` and `FromStr` traits are implemented
+    // on `Uid`, `Gid` and `Pid`.
+    #[test]
+    fn check_uid_gid_from_impls() {
+        use std::convert::TryFrom;
+        use std::str::FromStr;
+
+        assert!(crate::Uid::try_from(0usize).is_ok());
+        assert!(crate::Uid::from_str("0").is_ok());
+
+        assert!(crate::Gid::try_from(0usize).is_ok());
+        assert!(crate::Gid::from_str("0").is_ok());
+
+        assert!(crate::Pid::try_from(0usize).is_ok());
+        // If it doesn't panic, it's fine.
+        let _ = crate::Pid::from(0);
+        assert!(crate::Pid::from_str("0").is_ok());
+    }
 }
