@@ -1,8 +1,8 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
 use crate::{
-    sys::{component::Component, Disk, Networks, Process, Processor},
-    LoadAvg, Pid, ProcessRefreshKind, RefreshKind, SystemExt, User,
+    sys::{component::Component, Cpu, Disk, Networks, Process},
+    CpuRefreshKind, LoadAvg, Pid, ProcessRefreshKind, RefreshKind, SystemExt, User,
 };
 
 use std::cell::UnsafeCell;
@@ -11,10 +11,12 @@ use std::ffi::CStr;
 use std::mem::MaybeUninit;
 use std::path::{Path, PathBuf};
 use std::ptr::NonNull;
+use std::time::Duration;
 
 use super::utils::{
-    self, boot_time, c_buf_to_string, from_cstr_array, get_sys_value, get_sys_value_array,
-    get_sys_value_by_name, get_sys_value_str_by_name, get_system_info, init_mib,
+    self, boot_time, c_buf_to_string, from_cstr_array, get_frequency_for_cpu, get_sys_value,
+    get_sys_value_array, get_sys_value_by_name, get_sys_value_str_by_name, get_system_info,
+    init_mib,
 };
 
 use libc::c_int;
@@ -62,19 +64,21 @@ pub struct System {
     mem_used: u64,
     swap_total: u64,
     swap_used: u64,
-    global_processor: Processor,
-    processors: Vec<Processor>,
+    global_cpu: Cpu,
+    cpus: Vec<Cpu>,
     components: Vec<Component>,
     disks: Vec<Disk>,
     networks: Networks,
     users: Vec<User>,
     boot_time: u64,
     system_info: SystemInfo,
+    got_cpu_frequency: bool,
 }
 
 impl SystemExt for System {
     const IS_SUPPORTED: bool = true;
     const SUPPORTED_SIGNALS: &'static [Signal] = supported_signals();
+    const MINIMUM_CPU_UPDATE_INTERVAL: Duration = Duration::from_millis(100);
 
     fn new_with_specifics(refreshes: RefreshKind) -> System {
         let system_info = SystemInfo::new();
@@ -86,14 +90,15 @@ impl SystemExt for System {
             mem_used: 0,
             swap_total: 0,
             swap_used: 0,
-            global_processor: Processor::new(String::new(), String::new(), 0),
-            processors: Vec::with_capacity(system_info.nb_cpus as _),
+            global_cpu: Cpu::new(String::new(), String::new(), 0),
+            cpus: Vec::with_capacity(system_info.nb_cpus as _),
             components: Vec::with_capacity(2),
             disks: Vec::with_capacity(1),
             networks: Networks::new(),
             users: Vec::new(),
             boot_time: boot_time(),
             system_info,
+            got_cpu_frequency: false,
         };
         s.refresh_specifics(refreshes);
         s
@@ -110,40 +115,43 @@ impl SystemExt for System {
         self.swap_used = swap_used;
     }
 
-    fn refresh_cpu(&mut self) {
-        if self.processors.is_empty() {
-            let mut frequency: libc::size_t = 0;
+    fn refresh_cpu_specifics(&mut self, refresh_kind: CpuRefreshKind) {
+        if self.cpus.is_empty() {
+            let mut frequency = 0;
 
-            // We get the processor vendor ID in here.
+            // We get the CPU vendor ID in here.
             let vendor_id =
                 get_sys_value_str_by_name(b"hw.model\0").unwrap_or_else(|| "<unknown>".to_owned());
             for pos in 0..self.system_info.nb_cpus {
-                unsafe {
-                    // The information can be missing if it's running inside a VM.
-                    if !get_sys_value_by_name(
-                        format!("dev.cpu.{}.freq\0", pos).as_bytes(),
-                        &mut frequency,
-                    ) {
-                        frequency = 0;
+                if refresh_kind.frequency() {
+                    unsafe {
+                        frequency = get_frequency_for_cpu(pos);
                     }
                 }
-                self.processors.push(Processor::new(
-                    format!("cpu {}", pos),
-                    vendor_id.clone(),
-                    frequency as _,
-                ));
+                self.cpus
+                    .push(Cpu::new(format!("cpu {pos}"), vendor_id.clone(), frequency));
             }
-            self.global_processor.vendor_id = vendor_id;
+            self.global_cpu.vendor_id = vendor_id;
+            self.got_cpu_frequency = refresh_kind.frequency();
+        } else if refresh_kind.frequency() && !self.got_cpu_frequency {
+            for (pos, proc_) in self.cpus.iter_mut().enumerate() {
+                unsafe {
+                    proc_.frequency = get_frequency_for_cpu(pos as _);
+                }
+            }
+            self.got_cpu_frequency = true;
+        }
+        if refresh_kind.cpu_usage() {
+            self.system_info
+                .get_cpu_usage(&mut self.global_cpu, &mut self.cpus);
         }
-        self.system_info
-            .get_cpu_usage(&mut self.global_processor, &mut self.processors);
     }
 
     fn refresh_components_list(&mut self) {
-        if self.processors.is_empty() {
+        if self.cpus.is_empty() {
             self.refresh_cpu();
         }
-        self.components = unsafe { super::component::get_components(self.processors.len()) };
+        self.components = unsafe { super::component::get_components(self.cpus.len()) };
     }
 
     fn refresh_processes_specifics(&mut self, refresh_kind: ProcessRefreshKind) {
@@ -172,17 +180,7 @@ impl SystemExt for System {
 
             macro_rules! multi_iter {
                 ($name:ident, $($iter:tt)+) => {
-                    $name = crate::utils::into_iter(procs).$($iter)+.and_then(|kproc| {
-                        super::process::get_process_data(
-                            kproc,
-                            &proc_list,
-                            page_size,
-                            fscale,
-                            now,
-                            refresh_kind,
-                        )
-                        .map(|p| (kproc, p))
-                    });
+                    $name = crate::utils::into_iter(procs).$($iter)+;
                 }
             }
 
@@ -192,14 +190,25 @@ impl SystemExt for System {
             #[cfg(feature = "multithread")]
             multi_iter!(ret, find_any(|kproc| kproc.ki_pid == pid.0));
 
-            if let Some((kproc, proc_)) = ret {
-                self.add_missing_proc_info(self.system_info.kd.as_ptr(), kproc, proc_);
-                true
+            let kproc = if let Some(kproc) = ret {
+                kproc
             } else {
-                self.process_list
-                    .get(&pid)
-                    .map(|p| p.updated)
-                    .unwrap_or(false)
+                return false;
+            };
+            match super::process::get_process_data(
+                kproc,
+                &proc_list,
+                page_size,
+                fscale,
+                now,
+                refresh_kind,
+            ) {
+                Ok(Some(proc_)) => {
+                    self.add_missing_proc_info(self.system_info.kd.as_ptr(), kproc, proc_);
+                    true
+                }
+                Ok(None) => true,
+                Err(_) => false,
             }
         }
     }
@@ -232,12 +241,12 @@ impl SystemExt for System {
         &mut self.networks
     }
 
-    fn global_processor_info(&self) -> &Processor {
-        &self.global_processor
+    fn global_cpu_info(&self) -> &Cpu {
+        &self.global_cpu
     }
 
-    fn processors(&self) -> &[Processor] {
-        &self.processors
+    fn cpus(&self) -> &[Cpu] {
+        &self.cpus
     }
 
     fn physical_core_count(&self) -> Option<usize> {
@@ -297,6 +306,13 @@ impl SystemExt for System {
         &mut self.disks
     }
 
+    fn sort_disks_by<F>(&mut self, compare: F)
+    where
+        F: FnMut(&Disk, &Disk) -> std::cmp::Ordering,
+    {
+        self.disks.sort_unstable_by(compare);
+    }
+
     fn uptime(&self) -> u64 {
         unsafe {
             let csec = libc::time(::std::ptr::null_mut());
@@ -344,6 +360,10 @@ impl SystemExt for System {
     fn os_version(&self) -> Option<String> {
         self.system_info.get_os_release()
     }
+
+    fn distribution_id(&self) -> String {
+        std::env::consts::OS.to_owned()
+    }
 }
 
 impl Default for System {
@@ -367,10 +387,6 @@ impl System {
             #[cfg(not(feature = "multithread"))]
             use std::iter::Iterator as IterTrait;
 
-            crate::utils::into_iter(&mut self.process_list).for_each(|(_, proc_)| {
-                proc_.updated = false;
-            });
-
             let fscale = self.system_info.fscale;
             let page_size = self.system_info.page_size as isize;
             let now = super::utils::get_now();
@@ -387,13 +403,15 @@ impl System {
                     now,
                     refresh_kind,
                 )
-                .map(|p| (kproc, p))
+                .ok()
+                .and_then(|p| p.map(|p| (kproc, p)))
             })
             .collect::<Vec<_>>()
         };
 
         // We remove all processes that don't exist anymore.
-        self.process_list.retain(|_, v| v.updated);
+        self.process_list
+            .retain(|_, v| std::mem::replace(&mut v.updated, false));
 
         for (kproc, proc_) in procs {
             self.add_missing_proc_info(kd, kproc, proc_);
@@ -492,7 +510,7 @@ struct SystemInfo {
     mib_cp_times: [c_int; 2],
     // For the global CPU usage.
     cp_time: utils::VecSwitcher<libc::c_ulong>,
-    // For each processor CPU usage.
+    // For each CPU usage.
     cp_times: utils::VecSwitcher<libc::c_ulong>,
     /// From FreeBSD manual: "The kernel fixed-point scale factor". It's used when computing
     /// processes' CPU usage.
@@ -633,12 +651,15 @@ impl SystemInfo {
             }
             let swap =
                 std::slice::from_raw_parts(swap.as_ptr() as *mut libc::kvm_swap, nswap.min(LEN));
-            let (used, total) = swap.iter().fold((0, 0), |(used, total), swap| {
-                (used + swap.ksw_used as u64, total + swap.ksw_total as u64)
+            let (used, total) = swap.iter().fold((0, 0), |(used, total): (u64, u64), swap| {
+                (
+                    used.saturating_add(swap.ksw_used as _),
+                    total.saturating_add(swap.ksw_total as _),
+                )
             });
             (
-                used * self.page_size as u64 / 1_000,
-                total * self.page_size as u64 / 1_000,
+                used.saturating_mul(self.page_size as _),
+                total.saturating_mul(self.page_size as _),
             )
         }
     }
@@ -647,14 +668,14 @@ impl SystemInfo {
         let mut nb_pages: u64 = 0;
         unsafe {
             if get_sys_value(&self.virtual_page_count, &mut nb_pages) {
-                return nb_pages * self.page_size as u64 / 1_000;
+                return nb_pages.saturating_mul(self.page_size as _);
             }
 
             // This is a fallback. It includes all the available memory, not just the one available for
             // the users.
             let mut total_memory: u64 = 0;
             get_sys_value(&self.hw_physical_memory, &mut total_memory);
-            total_memory / 1_000
+            total_memory
         }
     }
 
@@ -666,14 +687,15 @@ impl SystemInfo {
             get_sys_value(&self.virtual_active_count, &mut mem_active);
             get_sys_value(&self.virtual_wire_count, &mut mem_wire);
 
-            let mut mem_wire = mem_wire * self.page_size as u64;
+            let mut mem_wire = mem_wire.saturating_mul(self.page_size as _);
             // We need to subtract "ZFS ARC" from the "wired memory" because it should belongs to cache
             // but the kernel reports it as "wired memory" instead...
             if let Some(arc_size) = self.zfs.arc_size() {
                 mem_wire -= arc_size;
             }
-            let used = (mem_active * self.page_size as u64) + mem_wire;
-            used / 1_000
+            mem_active
+                .saturating_mul(self.page_size as _)
+                .saturating_add(mem_wire)
         }
     }
 
@@ -689,37 +711,34 @@ impl SystemInfo {
             get_sys_value(&self.virtual_cache_count, &mut cached_mem);
             get_sys_value(&self.virtual_free_count, &mut free_mem);
             // For whatever reason, buffers_mem is already the right value...
-            let free = buffers_mem
-                + (inactive_mem * self.page_size as u64)
-                + (cached_mem * self.page_size as u64)
-                + (free_mem * self.page_size as u64);
-            free / 1_000
+            buffers_mem
+                .saturating_add(inactive_mem.saturating_mul(self.page_size as _))
+                .saturating_add(cached_mem.saturating_mul(self.page_size as _))
+                .saturating_add(free_mem.saturating_mul(self.page_size as _))
         }
     }
 
-    fn get_cpu_usage(&mut self, global: &mut Processor, processors: &mut [Processor]) {
+    fn get_cpu_usage(&mut self, global: &mut Cpu, cpus: &mut [Cpu]) {
         unsafe {
             get_sys_value_array(&self.mib_cp_time, self.cp_time.get_mut());
             get_sys_value_array(&self.mib_cp_times, self.cp_times.get_mut());
         }
 
-        fn fill_processor(
-            proc_: &mut Processor,
-            new_cp_time: &[libc::c_ulong],
-            old_cp_time: &[libc::c_ulong],
-        ) {
+        fn fill_cpu(proc_: &mut Cpu, new_cp_time: &[libc::c_ulong], old_cp_time: &[libc::c_ulong]) {
             let mut total_new: u64 = 0;
             let mut total_old: u64 = 0;
             let mut cp_diff: libc::c_ulong = 0;
 
             for i in 0..(libc::CPUSTATES as usize) {
-                // We obviously don't want to get the idle part of the processor usage, otherwise
+                // We obviously don't want to get the idle part of the CPU usage, otherwise
                 // we would always be at 100%...
                 if i != libc::CP_IDLE as usize {
                     cp_diff += new_cp_time[i] - old_cp_time[i];
                 }
-                total_new += new_cp_time[i] as u64;
-                total_old += old_cp_time[i] as u64;
+                let mut tmp: u64 = new_cp_time[i] as _;
+                total_new += tmp;
+                tmp = old_cp_time[i] as _;
+                total_old += tmp;
             }
 
             let total_diff = total_new - total_old;
@@ -730,13 +749,13 @@ impl SystemInfo {
             }
         }
 
-        fill_processor(global, self.cp_time.get_new(), self.cp_time.get_old());
+        fill_cpu(global, self.cp_time.get_new(), self.cp_time.get_old());
         let old_cp_times = self.cp_times.get_old();
         let new_cp_times = self.cp_times.get_new();
-        for (pos, proc_) in processors.iter_mut().enumerate() {
+        for (pos, proc_) in cpus.iter_mut().enumerate() {
             let index = pos * libc::CPUSTATES as usize;
 
-            fill_processor(proc_, &new_cp_times[index..], &old_cp_times[index..]);
+            fill_cpu(proc_, &new_cp_times[index..], &old_cp_times[index..]);
         }
     }
 
