@@ -7,6 +7,7 @@
 #![allow(clippy::upper_case_acronyms)]
 #![allow(clippy::non_send_fields_in_send_ty)]
 #![allow(renamed_and_removed_lints)]
+#![allow(clippy::assertions_on_constants)]
 #![allow(unknown_lints)]
 
 #[macro_use]
@@ -23,15 +24,18 @@ cfg_if::cfg_if! {
     } else if #[cfg(any(target_os = "macos", target_os = "ios"))] {
         mod apple;
         use apple as sys;
-        extern crate core_foundation_sys;
+        mod network_helper_nix;
+        use network_helper_nix as network_helper;
+        mod network;
 
         #[cfg(test)]
         pub(crate) const MIN_USERS: usize = 1;
     } else if #[cfg(windows)] {
         mod windows;
         use windows as sys;
-        extern crate winapi;
-        extern crate ntapi;
+        mod network_helper_win;
+        use network_helper_win as network_helper;
+        mod network;
 
         #[cfg(test)]
         pub(crate) const MIN_USERS: usize = 1;
@@ -39,6 +43,9 @@ cfg_if::cfg_if! {
         mod linux;
         use linux as sys;
         pub(crate) mod users;
+        mod network_helper_nix;
+        use network_helper_nix as network_helper;
+        mod network;
 
         #[cfg(test)]
         pub(crate) const MIN_USERS: usize = 1;
@@ -46,6 +53,9 @@ cfg_if::cfg_if! {
         mod freebsd;
         use freebsd as sys;
         pub(crate) mod users;
+        mod network_helper_nix;
+        use network_helper_nix as network_helper;
+        mod network;
 
         #[cfg(test)]
         pub(crate) const MIN_USERS: usize = 1;
@@ -59,12 +69,12 @@ cfg_if::cfg_if! {
 }
 
 pub use common::{
-    get_current_pid, DiskType, DiskUsage, Gid, LoadAvg, NetworksIter, Pid, PidExt,
-    ProcessRefreshKind, ProcessStatus, RefreshKind, Signal, Uid, User,
+    get_current_pid, CpuRefreshKind, DiskType, DiskUsage, Gid, LoadAvg, MacAddr, NetworksIter, Pid,
+    PidExt, ProcessRefreshKind, ProcessStatus, RefreshKind, Signal, Uid, User,
 };
-pub use sys::{Component, Disk, NetworkData, Networks, Process, Processor, System};
+pub use sys::{Component, Cpu, Disk, NetworkData, Networks, Process, System};
 pub use traits::{
-    ComponentExt, DiskExt, NetworkExt, NetworksExt, ProcessExt, ProcessorExt, SystemExt, UserExt,
+    ComponentExt, CpuExt, DiskExt, NetworkExt, NetworksExt, ProcessExt, SystemExt, UserExt,
 };
 
 #[cfg(feature = "c-interface")]
@@ -78,10 +88,10 @@ mod system;
 mod traits;
 mod utils;
 
-/// This function is only used on linux targets, on the other platforms it does nothing and returns
+/// This function is only used on Linux targets, on the other platforms it does nothing and returns
 /// `false`.
 ///
-/// On linux, to improve performance, we keep a `/proc` file open for each process we index with
+/// On Linux, to improve performance, we keep a `/proc` file open for each process we index with
 /// a maximum number of files open equivalent to half of the system limit.
 ///
 /// The problem is that some users might need all the available file descriptors so we need to
@@ -251,13 +261,31 @@ mod test {
             .processes()
             .iter()
             .all(|(_, proc_)| proc_.cpu_usage() >= 0.0
-                && proc_.cpu_usage() <= (s.processors().len() as f32) * 100.0));
+                && proc_.cpu_usage() <= (s.cpus().len() as f32) * 100.0));
         assert!(s
             .processes()
             .iter()
             .any(|(_, proc_)| proc_.cpu_usage() > 0.0));
     }
 
+    #[test]
+    fn check_cpu_usage() {
+        if !System::IS_SUPPORTED {
+            return;
+        }
+        let mut s = System::new();
+        for _ in 0..10 {
+            s.refresh_cpu();
+            // Wait a bit to update CPU usage values
+            std::thread::sleep(std::time::Duration::from_millis(100));
+            if s.cpus().iter().any(|c| c.cpu_usage() > 0.0) {
+                // All good!
+                return;
+            }
+        }
+        panic!("CPU usage is always zero...");
+    }
+
     #[test]
     fn check_users() {
         let mut s = System::new();
@@ -289,25 +317,31 @@ mod test {
                     .iter()
                     .find(|u| u.name() == "root")
                     .expect("no root user");
-                assert_eq!(*user.uid(), 0);
-                assert_eq!(*user.gid(), 0);
-                if let Some(user) = users.iter().find(|u| *u.gid() > 0) {
-                    assert!(*user.uid() > 0);
-                    assert!(*user.gid() > 0);
+                assert_eq!(**user.id(), 0);
+                assert_eq!(*user.group_id(), 0);
+                if let Some(user) = users.iter().find(|u| *u.group_id() > 0) {
+                    assert!(**user.id() > 0);
+                    assert!(*user.group_id() > 0);
                 }
+                assert!(users.iter().filter(|u| **u.id() > 0).count() > 0);
             }
-            #[cfg(not(target_os = "windows"))]
-            {
-                assert!(users.iter().filter(|u| *u.uid() > 0).count() > 0);
-            }
+
+            // And now check that our `get_user_by_id` method works.
+            s.refresh_processes();
+            assert!(s
+                .processes()
+                .iter()
+                .filter_map(|(_, p)| p.user_id())
+                .any(|uid| s.get_user_by_id(uid).is_some()));
         }
     }
 
     #[test]
     fn check_system_info() {
+        let s = System::new();
+
         // We don't want to test on unsupported systems.
         if System::IS_SUPPORTED {
-            let s = System::new();
             assert!(!s.name().expect("Failed to get system name").is_empty());
 
             assert!(!s
@@ -322,6 +356,8 @@ mod test {
                 .expect("Failed to get long OS version")
                 .is_empty());
         }
+
+        assert!(!s.distribution_id().is_empty());
     }
 
     #[test]
@@ -360,11 +396,11 @@ mod test {
     }
 
     #[test]
-    fn check_processors_number() {
+    fn check_cpus_number() {
         let mut s = System::new();
 
         // This information isn't retrieved by default.
-        assert!(s.processors().is_empty());
+        assert!(s.cpus().is_empty());
         if System::IS_SUPPORTED {
             // The physical cores count is recomputed every time the function is called, so the
             // information must be relevant even with nothing initialized.
@@ -373,20 +409,20 @@ mod test {
                 .expect("failed to get number of physical cores");
 
             s.refresh_cpu();
-            // The processors shouldn't be empty anymore.
-            assert!(!s.processors().is_empty());
+            // The cpus shouldn't be empty anymore.
+            assert!(!s.cpus().is_empty());
 
             // In case we are running inside a VM, it's possible to not have a physical core, only
             // logical ones, which is why we don't test `physical_cores_count > 0`.
             let physical_cores_count2 = s
                 .physical_core_count()
                 .expect("failed to get number of physical cores");
-            assert!(physical_cores_count2 <= s.processors().len());
+            assert!(physical_cores_count2 <= s.cpus().len());
             assert_eq!(physical_cores_count, physical_cores_count2);
         } else {
             assert_eq!(s.physical_core_count(), None);
         }
-        assert!(s.physical_core_count().unwrap_or(0) <= s.processors().len());
+        assert!(s.physical_core_count().unwrap_or(0) <= s.cpus().len());
     }
 
     #[test]
@@ -403,4 +439,62 @@ mod test {
             );
         }
     }
+
+    // Ensure that the CPUs frequency isn't retrieved until we ask for it.
+    #[test]
+    fn check_cpu_frequency() {
+        if !System::IS_SUPPORTED {
+            return;
+        }
+        let mut s = System::new();
+        s.refresh_processes();
+        for proc_ in s.cpus() {
+            assert_eq!(proc_.frequency(), 0);
+        }
+        s.refresh_cpu();
+        for proc_ in s.cpus() {
+            assert_eq!(proc_.frequency(), 0);
+        }
+        // In a VM, it'll fail.
+        if std::env::var("APPLE_CI").is_err() && std::env::var("FREEBSD_CI").is_err() {
+            s.refresh_cpu_specifics(CpuRefreshKind::everything());
+            for proc_ in s.cpus() {
+                assert_ne!(proc_.frequency(), 0);
+            }
+        }
+    }
+
+    // In case `Process::updated` is misused, `System::refresh_processes` might remove them
+    // so this test ensures that it doesn't happen.
+    #[test]
+    fn check_refresh_process_update() {
+        if !System::IS_SUPPORTED {
+            return;
+        }
+        let mut s = System::new_all();
+        let total = s.processes().len() as isize;
+        s.refresh_processes();
+        let new_total = s.processes().len() as isize;
+        // There should be almost no difference in the processes count.
+        assert!(
+            (new_total - total).abs() <= 5,
+            "{} <= 5",
+            (new_total - total).abs()
+        );
+    }
+
+    // We ensure that the `Process` cmd information is retrieved as expected.
+    #[test]
+    fn check_cmd_line() {
+        if !System::IS_SUPPORTED {
+            return;
+        }
+        let mut sys = System::new();
+        sys.refresh_processes_specifics(ProcessRefreshKind::new());
+
+        assert!(sys
+            .processes()
+            .iter()
+            .any(|(_, process)| !process.cmd().is_empty()));
+    }
 }
